#!/usr/bin/env python
# encoding: utf-8

from stat import S_IFREG
from fuse import fuse_get_context
from time import time


class File(object):

    def __init__(self):
        uid, gid, _ = fuse_get_context()
        self.attrs = {
            'st_mode': (S_IFREG | 0o755),
            'st_mtime': time(),
            'st_atime': time(),
            'st_nlink': 0,
            'st_uid': uid,
            'st_gid': gid,
            'st_size': 0
        }

    def get_attrs(self):
        return self.attrs



#!/usr/bin/env python
# encoding: utf-8

"""Class to handle assignment Moodle actions, it extends MoodleHandler
   and therefor is an observer of Moodle.
"""

from moodlefuse.moodle.assignments.assignment_grader import AssignmentGrader
from moodlefuse.moodle.assignments.assignment_parser import AssignmentParser
from moodlefuse.moodle.handler import MoodleHandler


class AssignmentHandler(MoodleHandler):

    def __init__(self, emulator, js_emulator):
        super(self.__class__, self).__init__(emulator, js_emulator)
        self.grader = AssignmentGrader()
        self.parser = AssignmentParser()

    def get_assignment_info_as_array(self):
        return ['Submissions', 'grades.csv']

    def get_assignment_submissions(self, assignment_url):
        if 'grading' not in assignment_url:
            assignment_url = assignment_url + "&action=grading"
        self.moodle.follow_link(assignment_url)
        assignment_submissions = self.moodle.filter_assignment_submissions()
        return self.parser.get_all_assignment_names(assignment_submissions)

    def fill_grades_file_with_template(self, location):
        csv = self.grader.format_csv(location)
        self.grader.add_user_names_and_emails(csv, None)
        return csv

    def get_grades_csv(self, location):
        return self.fill_grades_file_with_template(location)



#!/usr/bin/env python
# encoding: utf-8

"""Class to spin up a headless browser to handle Moodle interaction
"""

from mechanize import Browser, CookieJar
from moodlefuse.core import config
from bs4 import BeautifulSoup

from moodlefuse.moodle.resources import resource_errors
from moodlefuse.helpers import throws_moodlefuse_error
from moodlefuse.moodle.courses import course_errors
from moodlefuse.moodle import exception


class CoreEmulator(object):

    def __init__(self, username, password):
        self.username = username
        self.password = password
        self.setup_emulator()

    def setup_emulator(self):
        self.browser = Browser()
        self.browser.set_handle_robots(False)
        self.browser.addheaders = [
            ('User-Agent',
             'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 \
             (KHTML, like Gecko) Chrome/34.0.1847.132 Safari/537.36'),
            ('Accept', 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8'),
            ('Cache-Control', 'max-age=0'),
            ('Accept-Language', 'en-GB,en-US;q=0.8,en;q=0.6'),
            ('Connection', 'keep-alive')
        ]
        self.cookiejar = CookieJar()
        self.browser.set_cookiejar(self.cookiejar)

    @throws_moodlefuse_error(exception.NotFoundException)
    def open_login_page(self):
        if not config['MOODLE_WEB_ADDRESS'].endswith('php') and not config['MOODLE_WEB_ADDRESS'].endswith('html'):
            MOODLE_LOGIN_URL = config['MOODLE_WEB_ADDRESS'] + '/login/index.php'
        else:
            MOODLE_LOGIN_URL = config['MOODLE_WEB_ADDRESS']

        self.browser.open(MOODLE_LOGIN_URL)

    def login(self):
        self.open_login_page()
        self.browser.select_form(
            predicate=lambda form: form.attrs.get('id') == 'login'
        )
        self.browser.form.set_value(self.username, name='username')
        self.browser.form.set_value(self.password, name='password')
        resp = self.browser.submit()

        if resp.geturl().endswith('/login/index.php'):
            raise Exception

    @throws_moodlefuse_error(resource_errors.UnableToDownloadResource)
    def download(self, destination, source):
        source = str(source)
        if not source.startswith('http://') and not source.startswith('file://'):
            source = config['TEST_DATA'] + '/' + source

        self.browser.retrieve(source, destination)

    def open_link(self, url):
        response = self.browser.open(url)
        return BeautifulSoup(response.read())

    def check_form_checkbox(self, checkboxname):
        self.browser.find_control(checkboxname).items[0].selected = True

    def add_form_content(self, inputname, content):
        self.browser.form.set_value(content, name=inputname)

    def close_form(self):
        self.browser.submit()

    def get_current_url(self):
        return self.browser.geturl()

    def set_form_to_first_form(self):
        self.browser.select_form(nr=0)

    def set_form_to_form_with_control_value(self, value):
        for form in self.browser.forms():
            for control in form.controls:

                if control.value == value:
                    self.browser.form = form

    @throws_moodlefuse_error(exception.UnableToToggleEditing)
    def turn_course_editing_on(self):
        self.set_form_to_form_with_control_value('Turn editing on')
        response = self.browser.submit()
        return BeautifulSoup(response.read())

    def filter_assignment_submissions(self):
        self.browser.form = list(self.browser.forms())[2]
        self.browser.form["filter"] = ["submitted"]
        self.browser.form["perpage"] = ["100"]
        response = self.browser.submit()
        return BeautifulSoup(response.read())

    @throws_moodlefuse_error(exception.UnableToToggleEditing)
    def turn_course_editing_off(self):
        self.set_form_to_form_with_control_value('Turn editing off')
        response = self.browser.submit()
        return BeautifulSoup(response.read())

    @throws_moodlefuse_error(course_errors.InvalidMoodleIndex)
    def get_courses(self):
        return self.open_link(config['MOODLE_INDEX_ADDRESS'])

    @throws_moodlefuse_error(course_errors.UnableToObtainCategoryList)
    def get_course_categories(self, url):
        return self.open_link(url)

    @throws_moodlefuse_error(resource_errors.UnableToObtainResourceList)
    def get_course_resource_names(self, url):
        return self.open_link(url)

    def close(self):
        self.browser.close()



#!/usr/bin/env python
# encoding: utf-8

"""Derived class to parse Moodle course items, it parses
   Moodle and therefor extends off of Parser
"""

from moodlefuse.moodle.parser import Parser


class AssignmentParser(Parser):

    def __init__(self):
        super(AssignmentParser, self).__init__()

    def get_all_assignment_names(self, assignment_content):
        names = []
        names_list = self.scraper.get_html_items_with_tdclass(assignment_content, 'cell c2')
        for name_item in names_list:
            name = name_item.get_text()
            if name != '':
                name = name.replace(" ", "_")
                name = name + "_submission"
                names.append(name)

        return names



from tests.courses.data.course_results import true_courses, false_courses
from tests import MoodleFuseAppTestCase
from fuse import FuseOSError


class CoursesTestCase(MoodleFuseAppTestCase):

    def list_courses_test(self):
        files = list(
            self.ops.readdir(self.filesystem_root + '/', None))

        assert self.utils.lists_are_equal(files, true_courses)


    def list_courses_invalid_courses_test(self):
        files = list(
            self.ops.readdir(self.filesystem_root + '/', None))

        assert self.utils.lists_are_not_equal(files, false_courses)

    def enter_course_invalid_course_test(self):
        self.assertRaises(FuseOSError, lambda:
                self.ops.access(
                    self.filesystem_root + '/invalidcourse', None
                )
        )

    def get_coursee_attributes_test(self):
        attributes = self.ops.getattr(
            self.filesystem_root + '/testcourse'
        )

        self.assertTrue(
            (attributes.has_key(key)) for key in (
                'st_atime',
                'st_ctime',
                'st_gid',
                'st_mode',
                'st_mtime',
                'st_nlink',
                'st_size',
                'st_uid'
            )
        )

    def get_course_attributes_invalid_course_test(self):
        self.assertRaises(FuseOSError, lambda:
                self.ops.getattr(
                    self.filesystem_root + '/invalidcourse'
                )
        )




#!/usr/bin/env python
# encoding: utf-8

"""Class to handle assignment grading for lecturers.
"""

import csv

from moodlefuse.helpers import get_cache_path_based_on_location
from moodlefuse.core import config


class AssignmentGrader(object):

    def __init__(self):
        pass

    def format_csv(self, location):
        if 'DOWNLOADS' in config:
            csv_path = config['DOWNLOADS'] + '/grades.csv'
        else:
            csv_path = get_cache_path_based_on_location(location)
        with open(csv_path, 'wt') as f:
            writer = csv.writer(f)
            writer.writerow(('Name', 'Email Address', 'Grade'))
            f.close()
        return csv_path

    def add_user_names_and_emails(self, csv_path, assignment_content):
        with open(csv_path, 'a') as f:
            writer = csv.writer(f)
            writer.writerow(('test', 'test@test'))
            writer.writerow(('test1', 'test1@test1'))
            f.close()



#!/usr/bin/env python
# encoding: utf-8

"""Class to act as a bridge between the Moodle emulator and handlers
"""


class MoodleAPI(object):

    def __init__(self, emulator, js_emulator):
        self.emulator = emulator
        self.js_emulator = js_emulator

    def set_modify_moodle(self, modify=True):
        if modify is True:
            return self.emulator.turn_course_editing_on()
        else:
            return self.emulator.turn_course_editing_on()

    def follow_link(self, link):
        return self.emulator.open_link(link)

    def follow_link_with_js(self, link):
        return self.js_emulator.open_link(link)

    def get_current_url(self):
        return self.emulator.get_current_url()

    def get_resource_size(self, category, resource_name):
        self.js_emulator.turn_editing_on()
        self.js_emulator.open_edit_resource_menu(category, resource_name)
        size = self.js_emulator.get_resource_size()
        self.js_emulator.turn_editing_off()
        return size

    def filter_assignment_submissions(self):
        return self.emulator.filter_assignment_submissions()

    def add_new_resource(self, category, resource_name, resource_path):
        self.js_emulator.turn_editing_on()
        self.js_emulator.open_add_resource_menu(category)
        self.js_emulator.add_resource(resource_name, resource_path)
        self.js_emulator.turn_editing_off()

    def remove_existing_resource(self, category, resource_name):
        self.js_emulator.turn_editing_on()
        self.js_emulator.delete_resource(category, resource_name)
        self.js_emulator.turn_editing_off()

    def modify_existing_resource(self, category, resource_name, resource_path):
        self.js_emulator.turn_editing_on()
        self.js_emulator.open_edit_resource_menu(category, resource_name)
        self.js_emulator.edit_resource_content(resource_path)
        self.js_emulator.turn_editing_off()

    def rename_existing_resource(self, category, old_resource_name, new_resource_name):
        self.js_emulator.turn_editing_on()
        self.js_emulator.rename_file(category, old_resource_name, new_resource_name)
        self.js_emulator.turn_editing_off()

    def change_category_name(self, new_name):
        self.js_emulator.change_most_recent_categoryname(new_name)

    def download_resources(self, destination_link, source_link):
        self.emulator.download(destination_link, source_link)

    def get_courses(self):
        return self.emulator.get_courses()

    def get_course_contents(self, url):
        if url is not None:
            return self.emulator.get_course_categories(url)
        return None

    def close_api(self):
        self.js_emulator.close()
        self.emulator.close()



#!/usr/bin/env python
# encoding: utf-8

"""Derived class to parse Moodle course items, it parses
   Moodle and therefor extends off of Parser
"""

from moodlefuse.moodle.parser import Parser


class CourseParser(Parser):

    def __init__(self):
        super(CourseParser, self).__init__()

    def parse_courses(self, html):
        if self._html_is_table_format(html):
            return self._parse_table_courses(html)
        else:
            return self._parse_flat_courses(html)

    def _html_is_table_format(self, course_html):
        return self.scraper.get_html_with_divclass(
            course_html, 'courses frontpage-course-list-all') is None

    def _parse_flat_courses(self, course_html):
        course_html = self.scraper.get_html_with_divclass(
            course_html, 'courses frontpage-course-list-all')

        courses = course_html.findAll(text=True)
        return self.remove_unicode(courses)

    def _parse_table_courses(self, course_html):
        course_table = self.scraper.get_html_with_divclass(
            course_html, 'mymodules')

        courses_html = self.scraper.get_html_items_with_tdclass(
            course_table, 'cell c0')

        courses = self.scraper.get_text_from_html_list(courses_html)
        return self.remove_unicode(courses)

    def get_last_sections_edit_button(self, course_html):
        last_section_html = self.get_sections_settings_html(course_html)[-1]
        return self.get_link_from_item(last_section_html)

    def get_sections_settings_html(self, course_html):
        return self.scraper.get_all_html_with_atitle(
            course_html, 'Edit summary')

    def parse_course_category_titles(self, course_content):
        sections = self.scraper.get_text_from_taged_item(course_content, 'h3')
        return self.remove_unicode(sections)

    def parse_course_content_from_page(self, html):
        course_html = self.scraper.get_html_with_divclass(
            html, 'course-content')

        return course_html

    def get_add_section_link(self, course_content):
        add_section_html = self.scraper.get_html_with_aclass(
            course_content, 'increase-sections')

        return self.get_link_from_item(add_section_html)

    def parse_course_category_from_course(self, course_content, category):
        category_html = self.scraper.get_html_with_liarialabel(
            course_content, category
        )

        return category_html

    def get_course_link(self, course_scraper, course):
        course_link = self.scraper.get_link_from_linktext_in_divclass(
            course_scraper, 'courses frontpage-course-list-all', course)

        return course_link



#!/usr/bin/env python
# encoding: utf-8

"""Class to spin up a javscript enabled browser to handle Moodle
   interaction
"""

import time

from selenium import webdriver
from xvfbwrapper import Xvfb

from moodlefuse.moodle.resources import resource_errors
from moodlefuse.helpers import throws_moodlefuse_error
from moodlefuse.moodle.courses import course_errors
from moodlefuse.moodle import exception
from moodlefuse.core import config


class JsEmulator(object):

    def __init__(self, username, password):
        self.username = username
        self.password = password
        self.setup_emulator()

    def setup_emulator(self):
        self.vdisplay = Xvfb(width=1280, height=720)
        self.vdisplay.xvfb_cmd.append('-noreset')
        self.vdisplay.xvfb_cmd.append('-ac')
        self.vdisplay.start()
        self.driver = webdriver.Firefox()

    def open_add_resource_menu(self, category):
        element = self.driver.find_element_by_xpath("//li[@aria-label='{0}']".format(category))
        resource_menu = element.find_element_by_class_name('section-modchooser-text')
        resource_menu.click()

    def open_edit_resource_menu(self, category, resource_name):
        xpath = "//li[@aria-label='{0}']//span[contains(text(), '{1}')]".format(
            category, resource_name
        )
        element = self.driver.find_element_by_xpath(xpath)
        element = element.find_element_by_xpath("../../..")
        element.find_element_by_xpath(".//a[contains(text(), 'Edit')]").click()
        element.find_element_by_xpath(".//span[contains(text(), 'Edit settings')]").click()

    @throws_moodlefuse_error(resource_errors.UnableToRemoveFile)
    def delete_resource(self, category, resource_name):
        xpath = "//li[@aria-label='{0}']//span[contains(text(), '{1}')]".format(
            category, resource_name
        )
        element = self.driver.find_element_by_xpath(xpath)
        element = element.find_element_by_xpath("../../..")
        element.find_element_by_xpath(".//a[contains(text(), 'Edit')]").click()
        element.find_element_by_xpath(".//span[contains(text(), 'Delete')]").click()
        alert = self.driver.switch_to_alert()
        alert.accept()

    @throws_moodlefuse_error(resource_errors.UnableToRenameFile)
    def rename_file(self, category, old_name, new_name):
        self.open_edit_resource_menu(category, old_name)
        self.rename_file_from_edit_screen(new_name)
        self.driver.find_element_by_id("id_submitbutton2").click()

    def open_assignment_grading_and_submission_page(self):
        self.driver.find_element_by_xpath("//a[containts(text(), 'View/grade all submissions')]").click()

    def rename_file_from_edit_screen(self, new_name):
        time.sleep(.5)
        element = self.enter_text_into_textbox("id_name", new_name)
        element.send_keys(webdriver.common.keys.Keys.TAB)
        element = self.driver.switch_to.active_element
        element.click()
        element.send_keys(webdriver.common.keys.Keys.CONTROL, 'a')
        element.send_keys(new_name)

    @throws_moodlefuse_error(resource_errors.UnableToAddFile)
    def add_resource(self, resource_name, resource_path):
        self.driver.find_element_by_xpath("//span[contains(text(), 'File') and @class='typename']").click()
        self.driver.find_element_by_name("submitbutton").click()
        self.rename_file_from_edit_screen(resource_name)
        self.edit_resource_content(resource_path)
        self.driver.find_element_by_id("id_submitbutton2").click()

    @throws_moodlefuse_error(resource_errors.UnableToModifyFile)
    def edit_resource_content(self, resource_path):
        time.sleep(.5)
        self.driver.find_element_by_class_name('fp-mainfile').click()
        self.driver.find_element_by_class_name('fp-file-delete').click()
        self.driver.find_element_by_class_name('fp-dlg-butconfirm').click()
        self.driver.find_element_by_xpath("//div[@class='fp-btn-add']").click()
        element = self.driver.find_element_by_css_selector("input[type='file']")
        element.send_keys(resource_path)
        self.driver.find_element_by_class_name("fp-upload-btn").click()

    @throws_moodlefuse_error(course_errors.UnableToOAddCourseCategory)
    def change_most_recent_categoryname(self, new_name):
        self.check_form_checkbox('id_usedefaultname')
        self.enter_text_into_textbox('id_name', new_name)
        self.close_form()

    @throws_moodlefuse_error(exception.NotFoundException)
    def open_login_page(self):
        if not config['MOODLE_WEB_ADDRESS'].endswith('php') and not config['MOODLE_WEB_ADDRESS'].endswith('html'):
            MOODLE_LOGIN_URL = config['MOODLE_WEB_ADDRESS'] + '/login/index.php'
        else:
            MOODLE_LOGIN_URL = config['MOODLE_WEB_ADDRESS']
        self.driver.get(MOODLE_LOGIN_URL)

    @throws_moodlefuse_error(exception.LoginException)
    def login(self):
        self.open_login_page()
        element = self.driver.find_element_by_id("username")
        element.send_keys(self.username)
        element = self.driver.find_element_by_id("password")
        element.send_keys(self.password)
        self.driver.find_element_by_id("loginbtn").click()

    def open_link(self, url):
        self.driver.get(url)

    def check_form_checkbox(self, checkboxid):
        if self.driver.find_element_by_id(checkboxid).is_selected():
            self.driver.find_element_by_id(checkboxid).click()

    def enter_text_into_textbox(self, textbox, text):
        element = self.driver.find_element_by_id(textbox)
        element.clear()
        element.send_keys(text)
        return element

    @throws_moodlefuse_error(exception.UnableToToggleEditing)
    def turn_editing_on(self):
        element = self.driver.find_element_by_xpath("//* [@type='submit'][@value='Turn editing on']")
        element.click()

    @throws_moodlefuse_error(exception.UnableToToggleEditing)
    def turn_editing_off(self):
        element = self.driver.find_element_by_xpath("//* [@type='submit'][@value='Turn editing off']")
        element.click()

    def close_form(self):
        self.driver.find_element_by_id("id_submitbutton").click()

    def close(self):
        self.driver.quit()
        self.vdisplay.stop()



#!/usr/bin/env python
# encoding: utf-8

"""Class to configure MoodleFUSE
"""

import argparse
import getpass
import os

from alembic.config import Config as AlembicConfig
from ConfigParser import SafeConfigParser
from alembic import command

from moodlefuse.model_manager import setup_model
from moodlefuse.models.users import User
from moodlefuse.services import USERS


def main():
    Configurer()


class Configurer(object):

    def __init__(self):
        config_folder = self._create_config_folder()
        self._create_user_database()
        self._create_configuration(config_folder)
        self._create_filesystem_folder()
        self._create_file_cache()

    def _create_config_folder(self):
        config_folder = os.path.join(os.path.expanduser('~'), '.moodlefuse')
        if not os.path.exists(config_folder):
            os.makedirs(config_folder)
        os.chmod(config_folder, 0o700)
        return config_folder

    def _create_file_cache(self):
        cache_folder = os.path.join(
            os.path.expanduser('~'),
            '.moodlefuse/cache')
        if not os.path.exists(cache_folder):
            os.makedirs(cache_folder)
        os.chmod(cache_folder, 0o700)
        return cache_folder

    def _create_configuration(self, config_folder):
        args = self._generate_args()
        profile = args.pop('profile')
        config_file_path = config_folder + '/moodlefuse.conf'
        config = self._modify_config_profile(config_file_path, profile)
        self._add_user_to_database(config, profile)
        config_file = open(config_file_path, 'w+')
        config.write(config_file)

    def _create_user_database(self):
        database_file = os.path.join(
            os.path.expanduser('~'),
            '.moodlefuse/moodlefuse.sqlite'
        )

        if not os.path.exists(database_file):
            directory = os.path.join(os.path.dirname(__file__), '../alembic')
            config = AlembicConfig(os.path.join(
                directory,
                'alembic.ini'
            ))
            config.set_main_option('script_location', directory)
            command.upgrade(config, 'head', sql=False, tag=None)

        setup_model('sqlite:///' + database_file)

    def _generate_args(self):
        parser = argparse.ArgumentParser(
            'Command line utility for configuring moodlefuse'
        )

        parser.add_argument(
            '-p',
            '--profile',
            required=False,
            help='The profile to configure, default is initial',
            default='initial'
        )

        args = parser.parse_args()

        return vars(args)

    def _modify_config_profile(self, config_file, profile):
        config = SafeConfigParser()
        config.read(config_file)

        if not config.has_section(profile):
            config.add_section(profile)

        config = self._set_attributes_of_profile(config, profile)

        return config

    def _set_attributes_of_profile(self, config, profile):
        config = self._set_attribute_of_profile(
            config, profile, 'moodle_web_address',
            'Moodle server address',
            'http://www.moodle.com'
        )

        default_index = config.get(profile, 'moodle_web_address')

        config = self._set_attribute_of_profile(
            config, profile, 'moodle_index_address',
            'Moodle index address',
            default_index
        )

        config = self._set_attribute_of_profile(
            config, profile, 'local_moodle_folder',
            'Local Moodle folder name', 'moodle'
        )

        config = self._set_attribute_of_profile(
            config, profile, 'username',
            'Moodle Username', ''
        )

        return config

    def _get_hidden_password(self, found_user):
        return '*' * len(found_user.password)

    def _add_user_to_database(self, config, profile):
        username = config.get(profile, 'username')

        found_user = USERS.get(User, username)

        password = self._get_new_password_or_return_old(found_user)

        USERS.create(
            username=username,
            password=password
        )

    def _get_new_password_or_return_old(self, found_user):
        password_hidden = ''
        if found_user is not None:
            password_hidden = self._get_hidden_password(found_user)
            USERS.delete(found_user)

        password = self._read_in_config_attribute_or_use_default(
            'Moodle Password',
            password_hidden,
            getpass.getpass
        )

        if password == password_hidden and found_user is not None:
            password = found_user.password

        return password

    def _set_attribute_of_profile(self, config, profile, attribute, message, default):
        if config.has_option(profile, attribute):
            default = config.get(profile, attribute)

        attribute_value = self._read_in_config_attribute_or_use_default(
            message,
            default,
            raw_input
        )

        config.set(profile, attribute, attribute_value)

        return config

    def _read_in_config_attribute_or_use_default(self, message, default, method):
        attribute = method(message + ' [' + default + ']: ')
        if attribute == '':
            attribute = default

        return attribute

    def _create_filesystem_folder(self):
        config_folder = os.path.join(os.path.expanduser('~'), 'moodle')
        if not os.path.exists(config_folder):
            os.makedirs(config_folder)
        os.chmod(config_folder, 0o700)
        return config_folder



#!/usr/bin/env python
# encoding: utf-8

"""Class to carry out Moodle operations
"""


class RemoteHandler(object):

    def __init__(self, courses, resources, assignments):
        self.courses = courses
        self.resources = resources
        self.assignments = assignments

    def get_category_contents(self, location):
        course_contents = self.courses.enter_course_and_get_contents(location[0])
        return self.courses.get_course_category_contents(course_contents, location[1])

    def get_remote_resourse_names(self, location):
        if location[0] == '.Trash-1000':
            return None
        category_contents = self.get_category_contents(location)
        return self.resources.get_file_names_as_array(category_contents)

    def get_remote_grading_csv(self, location):
        return self.assignments.get_grades_csv(location)

    def get_remote_courses(self, location=None):
        return self.courses.get_courses_as_array()

    def get_remote_categories(self, location):
        return self.courses.get_course_categories_as_array(location[0])

    def add_category(self, location):
        self.courses.enter_course_and_get_contents(location[0])
        self.courses.add_new_category(location[1])

    def add_resource(self, location, path):
        self.courses.enter_course_with_js(location[0])
        self.resources.add_resource(path, location[1], location[2])

    def modify_resource(self, location, path):
        self.courses.enter_course_with_js(location[0])
        self.resources.modify_resource(path, location[1], location[2])

    def remove_resource(self, location):
        self.courses.enter_course_with_js(location[0])
        self.resources.remove_resource(location[1], location[2])

    def rename_resource(self, old_location, new_location):
        self.courses.enter_course_with_js(old_location[0])
        self.resources.rename_resource(old_location[1], old_location[2], new_location[3])

    def get_remote_assignment_names(self, location):
        category_contents = self.get_category_contents(location)
        assignment_url = self.resources.get_assignment_url(category_contents, location[2])
        return self.assignments.get_assignment_submissions(assignment_url)

    def get_remote_file_path(self, location):
        category_contents = self.get_category_contents(location)
        return self.resources.get_file_path(category_contents, location[2])

    def get_remote_assignment_path(self, location):
        category_contents = self.get_category_contents(location)
        return self.resources.get_assignment_url(category_contents, location[2])

    def get_remote_assignment_info(self, location=None):
        return self.assignments.get_assignment_info_as_array()

    def get_remote_assignment_submissions(self, location):
        assignment_url = self.get_remote_assignment_path(location)
        return self.assignments.get_assignment_submissions(assignment_url)

    def download_updated_file(self, location, moodle_url):
        return self.resources.download_resource(location, moodle_url)

    def is_valid_root(self, location=None):
        return True

    def is_valid_course(self, location):
        return location[0] in self.get_remote_courses()

    def is_valid_category(self, location):
        return location[1] in self.get_remote_categories(location)

    def is_valid_resource(self, location):
        return location[2] in self.get_remote_resourse_names(location)

    def is_valid_assignment(self, location):
        category_contents = self.get_category_contents(location)
        return self.resources.is_assignment(category_contents, location[2])

    def is_valid_assignment_info(self, location):
        return location[3] in self.get_remote_assignment_info()

    def is_valid_assignment_submission(self, location):
        return location[4] in self.get_remote_assignment_submissions(location)



#!/usr/bin/env python
# encoding: utf-8

true_courses = [
    '.' , '..',
    'testcourse',
    'testcourse2',
    'testcourse3'
]

false_courses = [
    '.' , '..',
    'testcourse2',
    'testcourse3'
]


#!/usr/bin/env python
# encoding: utf-8

"""Class to handle Exceptions relating to Moodle actions
"""

from moodlefuse.exception import MoodleFuseException


class MoodleException(MoodleFuseException):
    def __init__(self, specific_debug_info):
        debug_info = "Moodle action failed to complete - " + specific_debug_info
        super(MoodleException, self).__init__(debug_info)


class UnableToToggleEditing(MoodleException):
    def __init__(self):
        debug_info = "Could not toggle editing button"
        super(UnableToToggleEditing, self).__init__(debug_info)


class LoginException(MoodleException):
    def __init__(self):
        debug_info = "Failed to log into account for username provided in config"
        super(MoodleException, self).__init__(debug_info)


class NotFoundException(MoodleException):
    def __init__(self):
        debug_info = "Could not connect to the login URL provided in config"
        super(NotFoundException, self).__init__(debug_info)



#!/usr/bin/env python
# encoding: utf-8

"""This module creates a user service.
"""

from moodlefuse.model_manager import Service
from moodlefuse.models.users.models import User


class UsersService(Service):
    __model__ = User



#!/usr/bin/env python
# encoding: utf-8

import os

from setuptools import setup


def read_file(name):
    filepath = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        name
    )
    data = open(filepath)
    try:
        return data.read()
    except IOError:
        print "could not read %r" % name
        data.close()


PROJECT = 'moodlefuse'
VERSION = '0.0.2'
URL = 'http://github.com/brogand1993/MoodleFUSE'
AUTHOR = 'Darren Brogan'
AUTHOR_EMAIL = 'darren.brogan2@mail.dcu.ie'
DESC = "FUSE filesystem based on moodle implementation"
LONG_DESC = read_file('README.rst')
REQUIRES = [
    'requests',
    'fusepy',
    'mechanize',
    'selenium==2.45.0',
    'beautifulsoup4',
    'alembic',
    'sqlalchemy',
    'xvfbwrapper'
]

setup(
    name=PROJECT,
    version=VERSION,
    description=DESC,
    long_description=LONG_DESC,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    license='MIT License',
    package_data={
        'alembic': ['*.mako', 'versions/*']
    },
    packages=[
        'tests',
        'alembic',
        'moodlefuse',
        'tests.data',
        'tests.courses',
        'tests.resources',
        'tests.courses.data',
        'tests.resources.data',
        'moodlefuse.moodle',
        'moodlefuse.models',
        'moodlefuse.filesystem',
        'moodlefuse.models.users',
        'moodlefuse.moodle.courses',
        'moodlefuse.moodle.emulator',
        'moodlefuse.moodle.resources',
        'moodlefuse.filesystem.files',
        'moodlefuse.moodle.assignments',
    ],
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIRES,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Console',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities',
        'Programming Language :: Python :: 2.7',
    ],
    entry_points="""
        [console_scripts]
        moodlefuse = moodlefuse.__main__:main
        moodlefuse-configure = moodlefuse.configure:main
    """
)



#!/usr/bin/env python
# encoding: utf-8

"""Class to handle Exceptions relating to MoodleFUSE actions
"""


class MoodleFuseException(Exception):
    def __init__(self, debug_info):
        exception_reason = "ERROR ENCOUNTERED: MoodleFUSE has encountered an error."
        debug_info = debug_info
        self.message = exception_reason + debug_info

    def __str__(self):
        return self.message



#!/usr/bin/env python
# encoding: utf-8

from tests import MoodleFuseAppTestCase
from tests.data.settings import DOWNLOADS

import os


class FilesystemTestCase(MoodleFuseAppTestCase):

    def cache_created_test(self):
        self.assertTrue(os.path.exists(DOWNLOADS))

    def mount_created_test(self):
        self.assertTrue(os.path.exists(self.filesystem_root))


#!/usr/bin/env python
# encoding: utf-8

"""Class to scrape Moodle HTML, looking for specific
   information.
"""


class Scraper(object):

    def get_link_from_linktext_in_divclass(self, html, divclass, linktext):
        div_html = self.get_html_with_divclass(html, divclass)
        return self.get_link_from_linktext(div_html, linktext)

    def get_link_from_linktext(self, html, linktext):
        link = html.find('a', href=True, text=linktext)

        if link is not None:
            return link['href']

        return link

    def get_html_with_divclass(self, html, classname):
        return self._get_html_from_tag_label_with_name(
            html, 'div',
            'class', classname
        )

    def get_html_items_with_divclass(self, html, classname):
        return self._get_all_html_from_tag_label_with_name(
            html, 'div',
            'class', classname
        )

    def get_html_with_aclass(self, html, classname):
        return self._get_html_from_tag_label_with_name(
            html, 'a',
            'class', classname
        )

    def get_all_html_with_atitle(self, html, titlename):
        return self._get_all_html_from_tag_label_with_name(
            html, 'a',
            'title', titlename
        )

    def get_html_items_with_tdclass(self, html, classname):
        return self._get_all_html_from_tag_label_with_name(
            html, 'td',
            'class', classname
        )

    def get_html_items_with_spanclass(self, html, classname):
        return self._get_all_html_from_tag_label_with_name(
            html, 'span',
            'class', classname
        )

    def get_html_with_liarialabel(self, html, labelname):
        return self._get_html_from_tag_label_with_name(
            html, 'li',
            'aria-label', labelname
        )

    def get_instances_from_span_list(self, span_list):
        instances = []

        for span_item in span_list:
            sections = span_item.get_text().split(" ")
            instances.append(sections[0])

        return instances

    def get_link_from_span_list_with_type_and_name(self, spanlist, instance_type, name):
        for span_item in spanlist:
            sections = span_item.get_text().split(" ")
            if sections[1] == instance_type and sections[0] == name:
                return span_item.find('a', href=True)['href']

        return None

    def _get_html_from_tag_label_with_name(self, html, tag, label, name):
        if html is None:
            return None

        html = html.find(tag, attrs={
            label: name
        })

        return html

    def _get_all_html_from_tag_label_with_name(self, html, tag, label, name):
        if html is None:
            return None

        html = html.findAll(tag, attrs={
            label: name
        })

        return html

    def get_text_from_taged_item(self, html, tag):
        tagged_html = html.select(tag)
        texts = []
        for tag in tagged_html:
            texts.append(tag.get_text())

        return texts

    def get_text_from_html_list(self, html_list):
        texts = []
        for item in html_list:
            texts.append(item.get_text())

        return texts



#!/usr/bin/env python
# encoding: utf-8

"""Base class to parse Moodle items
"""

from moodlefuse.moodle.scraper import Scraper


class Parser(object):

    def __init__(self):
        self.scraper = Scraper()

    def remove_unicode(self, items):
        return [item.encode('utf-8') for item in items]

    def get_link_from_item(self, item):
        if item is not None:
            return item['href']

        return None
